<?php

namespace Drupal\phone_link\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'phone_link_field_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "phone_link_field_formatter",
 *   label = @Translation("Phone link"),
 *   field_types = {
 *     "string",
 *     "telephone"
 *   }
 * )
 */
class PhoneLinkFieldFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'title' => t('Call to @phone'),
        'text'  => '',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements['title'] = [
      '#type'          => 'textfield',
      '#title'         => t('Title tip'),
      '#description'   => t('Provide "title" HTML-attribute for phone link. You can use "@phone" replacement (without quotes).'),
      '#default_value' => $this->getSetting('title'),
    ];

    $elements['text'] = [
      '#type'          => 'textfield',
      '#title'         => t('Replace phone number'),
      '#description'   => t('Text displayed instead of phone. You can use "@phone" replacement (without quotes).'),
      '#default_value' => $this->getSetting('text'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $settings = $this->getSettings();

    if (!empty($settings['title'])) {
      $summary[] = t('Title attribute: @title', ['@title' => $settings['title']]);
    }
    else {
      $summary[] = t('No title attribute.');
    }

    if (!empty($settings['text'])) {
      $summary[] = t('Text: @text', ['@text' => $settings['text']]);
    }
    else {
      $summary[] = t('No text replacement.');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      $elements[$delta] = $this->viewValue($item);
    }

    return $elements;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    $phone = $item->value;
    $setting_title = t($this->getSetting('title'), ['@phone' => $phone]);
    $setting_text = t($this->getSetting('text'), ['@phone' => $phone]);

    $clean_phone = preg_replace('/[^\d+]/', '', $phone);

    $url = Url::fromUri('tel:' . substr($clean_phone, 0, 13));
    $link = Link::fromTextAndUrl($setting_text ?: $phone, $url)->toRenderable();
    $link['#options']['attributes']['title'] = $setting_title ?: $setting_title;
    $link['#options']['attributes']['class'][] = 'phone-link';

    return $link;
  }
}
